/*REPLACE_MARKER*/
/*DO NOT DELETE THESE COMMENTS*/

let oldOpenDateTimeCmd = GC.Spread.Sheets.Commands.openDateTimePicker.execute;

GC.Spread.Sheets.Commands.openDateTimePicker.execute = function(){
    let options = arguments[1];
    // only even num row will open the date time picker
    if(options.row%2 && options.col == 5){
        return oldOpenDateTimeCmd.apply(this, arguments);
    }
    return false;
}

window.onload = function() {
    var spread = new GC.Spread.Sheets.Workbook(document.getElementById("ss"), { sheetCount: 0 });
    initSpread(spread);
};

function initSpread(spread) {
    spread.suspendPaint();
    spread.options.autoFitType = GC.Spread.Sheets.AutoFitType.cellWithHeader;
    spread.clearSheets();

    var dataManager = spread.dataManager();

    initDefineOrderTable(spread, dataManager);

    spread.resumePaint();

}

function initDefineOrderTable(spread, dataManager) {
    spread.options.allowDynamicArray = true;

    var apiUrl = getApiUrl("DefineOrder"), apiColumnUrl = getColumnApiUrl("DefineOrder");
    var orderTable = dataManager.addTable("orderTable", {
        remote: {
            read: {
                url: apiUrl,
            },
            update: {
                url: apiUrl,
                method: 'PUT'
            },
            create: {
                url: apiUrl,
            },
            delete: {
                url: apiUrl,
            },
            getColumns: {
                url: apiColumnUrl
            },
            addColumn: {
                url: apiColumnUrl,
                method: 'POST'
            },
            updateColumn: {
                url: apiColumnUrl,
                method: 'PUT'
            },
            removeColumn: {
                url: apiColumnUrl,
                method: 'DELETE'
            },
            // batch: {
            //     url: apiUrl + 'Collection'
            // }
        },
        schema: {
            // It could define some options for the existed column on the table
            columns: {
                Id: { isPrimaryKey: true },
                OrderDate: { dataType: 'date' },
                RequiredDate: { dataType: 'date' },
                ShippedDate: { dataType: 'date' },
                ShipVia: {
                    dataType: 'number',
                    type: 'Select',
                    style: {
                        cellType: {
                            type: 'combobox',
                            editorValueType: 'value',
                            items: [
                                { text: 'Speedy Express', value: 1 },
                                { text: 'United Package', value: 2 },
                                { text: 'Federal Shipping', value: 3 }
                            ]
                        }
                    },
                },
                Freight: {
                    dataType: 'number',
                    type: 'Currency',
                    style: {
                        formatter: '[$$-409]#,##0.00'
                    },
                },
                CreatedTime: {
                    type: "CreatedTime",
                    dataType: 'date',
                    trigger: {
                        when: "onNew",
                        formula: "=NOW()"
                    },
                    readonly: true,
                    defaultValue: '=NOW()',
                    style: {
                        formatter: "m/d/yy h:mm;@"
                    },
                },
                ModifiedTime: {
                    type: "ModifiedTime",
                    dataType: 'date',
                    trigger: {
                        when: "onNewAndUpdate",
                        formula: "=NOW()",
                        fields: "*"
                    },
                    readonly: true,
                    style: {
                        formatter: "[$-409]m/d/yy h:mm AM/PM;@"
                    },
                },
                OrderAttachment: {
                    type: 'Attachment',
                    style: {
                        cellType: {
                            type: 'fileUpload'
                        }
                    },
                },
                OrderCode: {
                    type: 'Barcode',
                    defaultValue: '=FLOOR.MATH(RAND()*100000000)',
                    style: {
                        formatter: '=BC_GS1_128([@OrderCode],"#000000","#FFFFFF",false,,,,,,,,,,,)'
                    },
                },
            }
        },
        autoSync: true,
        // batch: true,
    });
    var employeeTable = dataManager.addTable("employeeTable", {
        remote: {
            read: {
                url: getApiUrl("Employee")
            }
        },
        schema: {
            columns: {
                Id: { isPrimaryKey: true },
                BirthDate: { dataType: 'date' },
                HireDate: { dataType: 'date' },
            }
        },
        autoSync: true
    });
    var customerTable = dataManager.addTable("customerTable", {
        remote: {
            read: {
                url: getApiUrl("Customer")
            }
        },
        schema: {
            columns: {
                Id: { isPrimaryKey: true },
            }
        },
        autoSync: true
    });
    dataManager.addRelationship(orderTable, "EmployeeId", "employee", employeeTable, "Id", "orders");
    dataManager.addRelationship(orderTable, "CustomerId", "customer", customerTable, "Id", "orders");

    var tableSheet = spread.addSheetTab(0, "Orders", GC.Spread.Sheets.SheetType.tableSheet);
    tableSheet.options.allowAddNew = true;
    tableSheet.options.enableDefineColumn = true;

    var rowActions = GC.Spread.Sheets.TableSheet.BuiltInRowActions;
    var options = tableSheet.rowActionOptions();
    options.push(
        rowActions.removeRow,
        rowActions.saveRow,
        rowActions.resetRow,
    );
    tableSheet.rowActionOptions(options);
    
    var orderView = orderTable.addView("orderView", [
        { value: "Id", width: 50 },
        { value: "CustomerId", defaultValue: 'ALFKI', visible: false }, // just for demo
        { value: "EmployeeId", defaultValue: 1, visible: false }, // just for demo
        { value: "customer.ContactName", caption: 'Contact', width: 100 },
        { value: '=CONCAT([@employee.FirstName], " ", [@employee.LastName])', caption: 'Employee', width: 110 },
        { value: "OrderDate", width: 100, style: { formatter: "m/d/yyyy" } },
        { value: "ShipVia", width: 140 },
        { value: "Freight", width: 80, defaultValue: 0 },
        { value: "ShipName" },
        { value: '=CONCAT([@ShipState], ", ", [@ShipCity], ", ", [@ShipRegion])', caption: 'Ship State', visible: false },
        { value: 'OrderAttachment', caption: 'Attachment', visible: false },
        { value: 'OrderCode', caption: 'Logistic Code', width: 160, visible: false },
        { value: 'CreatedTime', caption: 'Created Time', width: 160, visible: false },
        { value: 'ModifiedTime', caption: 'Modified Time', width: 160, visible: false },
    ], null, { defaultColumnWidth: 120 });
    orderView.fetch().then(function () {
        tableSheet.setDataView(orderView);
    });
}
function getBaseApiUrl() {
    return 'https://developer.mescius.com/spreadjs/demos/features/table-sheet/define-column/overview/purejs'.match(/http.+spreadjs\/demos\//)[0] + 'server/api';
}
function getApiUrl(tableName) {
    return getBaseApiUrl() + "/" + tableName;
}
function getColumnApiUrl(tableName) {
    return getBaseApiUrl() + "/tables/" + tableName + "/columns";
}